#include "config.h"
#include <cctype>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <strstream>

CConfig::ENUM_REPLIES CConfig::Parse( char *psz ) {
	char *p = psz;
	string Name("");
	Variable.String = "";
	
#ifndef SILENT_PARSER
	cout << "Parsing: " << psz << endl;
#endif

	// Eat leading spaces
	while (*p && isspace(*p)) p++;
	
	// Read name
	while (*p && !isspace(*p)) {
		Name += *p;
		p++;
	}


	// Eat whitespace
	while (*p && isspace(*p)) p++;

	// Read value
	while (*p) {
		Variable.String += *p;
		p++;
	}

#ifndef SILENT_PARSER
	cout << "Name.size(): " << Name.size() << endl;
	cout << "Variable.String.size(): " << Variable.String.size() << endl;
#endif

	if (Name.size() == 0) return NO_NAME;

	// TODO: Optimize
	if (Variable.String.size() == 0) {
		Latest.Int = GetInt( Name );
		Latest.Float = GetFloat( Name );
		Latest.String = GetString( Name );
		return NO_VALUE;
	}

	Variable.Int = atoi( Variable.String.c_str() );
	Variable.Float = atof( Variable.String.c_str() );

	// Delete previous variable with same name
	if ((Iterator = Variables.find( Name )) != Variables.end()) Variables.erase(Iterator);
	// Insert new variable
	Variables.insert( MYMAP::value_type( Name, Variable ) );

	return OK;
}

string CConfig::GetString( string &Name ) {
	if (GetVarByName( Name )) return Variable.String;
	return string("undefined");
}

int CConfig::GetInt( string &Name ) {
	if (GetVarByName( Name )) return Variable.Int;
	return 0;
}

float CConfig::GetFloat( string &Name ) {
	if (GetVarByName( Name )) return Variable.Float;
	return 0.0f;
}

bool CConfig::GetVarByName( string &Name ) {
	Iterator = Variables.find( Name );
	if (Iterator == Variables.end()) return false;
	Variable = (*Iterator).second;
	return true;
}

void CConfig::ParseCfg( char *File ) {
	ifstream istr(File);
	char Buffer[256];

	if (!istr.is_open()) {
		return;
	}
	while (!istr.eof()) {
		istr.getline( Buffer, 256 );
		if (Buffer[0] == '#') continue;
		if (Buffer[0] == 0x27) continue;
		Parse( Buffer );
	}
}

// This function silently ignores sections!
void CConfig::ParseIni( char *File ) {
	ifstream istr(File);
	char Buffer[256];
	int i = 0;

	if (!istr.is_open()) {
		return;
	}
	while (!istr.eof()) {
		istr.getline( Buffer, 256 );
		if (Buffer[0] == ';') continue;
		if (Buffer[0] == '[') continue;
		while (Buffer[i]) {
			if (Buffer[i] == '=') Buffer[i] = ' ';
			i++;
		}
		Parse( Buffer );
	}
}

void CConfig::ParseCgi() {
	char Buffer[4096], b[3]={0,0,0};
	int i = 0;
	string Method, Query;
	const string POST("POST"), GET("GET");
	istream *istr = NULL;
	
	// Environment variables involved:
	// REQUEST_METHOD, CONTENT_LENGTH, QUERY_STRING

	Method = getenv( "REQUEST_METHOD" );
	if ( Method == POST ) {
		istr = &cin;
	}
	if ( Method == GET ) {
		istrstream istrQuery( getenv( "QUERY_STRING" ) );
		istr = &istrQuery;
	}

	if (!istr) return;

	while ( !istr->eof() ) {
		istr->getline( Buffer, 4096, '&' );
		while ( Buffer[i] ) {
			if (Buffer[i] == '=') Buffer[i] = ' ';
			if (Buffer[i] == '+') Buffer[i] = ' ';
			if (Buffer[i] == '%') {
				// Hexadecimal encoding
				b[0] = Buffer[i+1];
				b[1] = Buffer[i+2];
				Buffer[i+1] = ' ';
				Buffer[i+2] = 0x08; // Should be backspace
				Buffer[i] = atoi( b );
			}
			i++;
		}
		Parse( Buffer );
	}
}

